#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

from ._public import *


def gen_machine_code(crypt=CRYPT, to_file=True) :
    '''
    用户场景： 生成机器码，并写入文件
    （可直接把文件发送给管理员，让其生成注册码）
    [param] crypt: 加解密类
    [param] to_file: 是否把用户码写入文件
    [return] 机器码码
    '''
    uuid = MI.generate()
    machine_code = crypt.encrypt_des(uuid)
    if to_file :
        save(machine_code, MACHINE_CODE_PATH)
    return machine_code


def read_user_code() :
    '''
    用户场景： 读取（或输入）被管理员分配的用户码
    [return] 用户码
    '''
    user_code = read(USER_CODE_PATH)   # 若无法从文件中读取
    if user_code == '' :
        user_code = input('请输入用户码: ')   # 则要求用户输入
    return user_code


def verify_authorization(user_code, crypt=CRYPT) :
    '''
    用户场景： 每次运行程序时，
        1. 输入用户码 
        2. 获取机器码
        3. 用户码 + 机器码 生成 注册码
        4. 读取管理员提供的 注册码
        5. 比较两个注册码是否相同
    [param] user_code: 用户码
    [param] crypt: 加解密类
    [return] true: 注册码一致； false: 注册码不同
    '''
    rst = False
    try :
        uuid = MI.generate()
        expire_time = crypt.decrypt_des(user_code)
        register_code = gen_rc(crypt, uuid, expire_time)
        rst = (register_code == read(REGISTER_CODE_PATH))
        if rst :
            expire_time = int(expire_time)
            expire_date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(expire_time))
            if expire_time == 0 :
                print('注册码正确（永久）。')

            elif now() <= expire_time :
                print('注册码正确（有效期至 %s）。' % expire_date)
            else :
                rst = False
                print('注册码已过期（有效期至 %s）。' % expire_date)
    except :
        pass
    if not rst :
        print('用户码错误 或 注册码不存在，请联系管理员。')
    return rst



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

import hashlib
import base64
from pyDes import *
import binascii
from ._tools import *


def gen_des_key() :
    '''
    生成 DES Key 、
    **生成后需注意自行保存，不要丢失**
    [return] DES Key
    '''
    des_key = gen_random_bytes()
    print("DES Key: %s" % des_key)
    return des_key


def gen_des_iv() :
    '''
    生成 DES IV （初始向量）。
    **生成后需注意自行保存，不要丢失**
    [return] DES IV
    '''
    des_iv = gen_random_bytes()
    print("DES IV: %s" % des_iv)
    return des_iv



class Crypt :

    def __init__(self, key=None, iv=None):
        self.des_key = key or b"FV>0Y_:D"                           # DES 加解密 Key
        self.des_iv = iv or b"\x34\x6A\x52\xBC\x37\x2D\x36\xEF"     # 初始 IV 向量（盐）
        self.des_obj = des(self.des_key, CBC, self.des_iv, pad=None, padmode=PAD_PKCS5)


    def encrypt_des(self, plaintext):
        '''
        DES 加密
        '''
        _des = self.des_obj.encrypt(plaintext)      # DES 加密
        _base64 = base64.b64encode(_des)            # DES密文 转为 Base64
        _hex = binascii.b2a_hex(_base64)            # Base64 转为 16进制字节
        ciphertext = b2s(_hex)                     # 16进制字节 转为 16进制字符串
        return ciphertext.upper()                   # 字符串大写
    

    def decrypt_des(self, ciphertext):
        '''
        DES 解密
        '''
        _hex = s2b(ciphertext)                     # 字符串 还原为 16进制字节
        _base64 = binascii.a2b_hex(_hex)            # 16进制字节 还原为 Base64
        _des = base64.b64decode(_base64)            # Base64 还原为 DES密文
        plaintext = self.des_obj.decrypt(_des)      # DES密文 解密为 明文字节
        return b2s(plaintext)                      # 明文字节 转为 明文字符串
    
    
    def to_md5(self, text) :
        '''
        生成 MD5
        '''
        return hashlib.md5(
            s2b(text)
        ).hexdigest().upper()


    




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

import random
import string


def gen_random_bytes() :
    '''
    生成 8 bit 的随机字节
    [return] 8 bit 随机字节
    '''
    str_range = "%s%s" % (string.ascii_letters, string.digits)
    return s2b(''.join(random.sample(str_range, 8)))


def b2s(value) :
    '''
    byte 转 str。
    [param] value: byte 字节
    [return] str 字符串
    '''
    if isinstance(value, bytes) :
        value = bytes.decode(value)
    return value


def s2b(value) :
    '''
    str 转 byte。
    [param] str 字符串
    [return] byte 字节
    '''
    if isinstance(value, str) :
        value = str.encode(value)
    return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

import os
import time
import requests
import json
from .crypt import Crypt
from ._mi import MachineInfo

CRYPT = Crypt()     
MI = MachineInfo()

JOINER = '##'
ONLINE_TIME_URL = 'http://api.m.taobao.com/rest/api3.do?api=mtop.common.getTimestamp'
UNLIMIT = '0000000000'

MACHINE_CODE_PATH = '.machine'
USER_CODE_PATH = '.user'
REGISTER_CODE_PATH = '.register'
CODE_PATHS = [ 
    MACHINE_CODE_PATH, 
    USER_CODE_PATH, 
    REGISTER_CODE_PATH 
]


def gen_rc(crypt, uuid, user_code) :
    '''
    生成注册码
    [param] crypt: 加解密类
    [param] uuid: 用户机器唯一表示
    [param] user_code: 用户码
    [return] 注册码
    '''
    return crypt.to_md5(
        crypt.encrypt_des(
            "%s%s%s" % (user_code, JOINER, uuid)
        )
    )


def now() :
    '''
    获取当前时间点（精确到秒）
    [return] 当前时间戳（long）
    '''
    try :
        # 先获取在线时间，避免用户修改本地时间绕过注册校验
        response = requests.get(ONLINE_TIME_URL, verify=False, timeout=10)
        millis = int(json.loads(response.text)['data']['t'])
        seconds = int(millis / 1000)
    except :
        # 网络不通时用本地时间代替
        seconds = int(time.time())
    return seconds


def after(days) :
    '''
    获取今天之后的 n 天的时间点（精确到秒）
    [param] days: 之后的 n 天
    [return] n 天后的时间戳（str）
    '''
    timestamp = UNLIMIT
    if days > 0 :
        seconds = days * 86400
        timestamp = now() + seconds
    return str(timestamp)



def save(code, filepath) :
    '''
    保存 xx码
    [param] code: xx码
    [param] filepath: 保存位置
    [return] xx码
    '''
    with open(filepath, 'w+') as file :
        file.write(code)
    return code


def read(filepath) :
    '''
    读取 xx码
    [param] filepath: 保存位置
    [return] xx码
    '''
    code = ''
    if os.path.exists(filepath) :
        with open(filepath, 'r') as file :
            code = file.read()

    # 若当前目录读取不到，则试图从上级目录读取
    # 用于使用 pyinstaller 发布后，需要创建“快捷方式”或“软链”到上层目录的场景
    else :
        parent_filepath = "../%s" % os.path.basename(filepath)
        if os.path.exists(parent_filepath) :
            with open(parent_filepath, 'r') as file :
                code = file.read()
    return code





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

import random
import string
from ._public import *


def read_machine_code() :
    '''
    管理员场景： 读取用户提供的机器码
    [return] 机器码
    '''
    return read(MACHINE_CODE_PATH)


def gen_user_code(days=3, crypt=CRYPT, to_file=True) :
    '''
    管理员场景： 分配用户码，并写入文件
    （可直接把文件发送给用户，让其放到程序根目录）
    [param] day: 授权天数。 0 为永久授权
    [param] to_file: 是否把用户码写入文件
    [return] 用户码
    '''
    after_time = after(int(days))
    user_code = crypt.encrypt_des(after_time)
    if to_file :
        save(user_code, USER_CODE_PATH)
    return user_code



def gen_register_code(machine_code, user_code, crypt=CRYPT, to_file=True) :
    '''
    管理员场景：
        1. 用户提供 机器码（加密）
        2. 用户预先指定 或 管理员随机分配 的用户码
        3. 用户码（解密） + 机器码 生成 注册码
        4. 注册码写入文件（可直接把文件发送给用户，让其放到程序根目录）
    [param] machine_code: 机器码
    [param] user_code: 用户码
    [param] crypt: 加解密类
    [param] to_file: 是否把机器码写入文件
    [return] 注册码
    '''
    try :
        uuid = crypt.decrypt_des(machine_code)
        expire_time = crypt.decrypt_des(user_code)
        register_code = gen_rc(crypt, uuid, expire_time)
    except :
        register_code = ''
        print('无法解密【机器码】：加密密钥不同、或加密格式不正确')
    if to_file :
        save(register_code, REGISTER_CODE_PATH)
    return register_code




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------

import platform
from psutil import net_if_addrs
from datetime import datetime


class MachineInfo :

    def __init__(self) -> None:
        pass


    def generate(self) :
        mc = '\n'.join([
            self._get_disk_uuid(), 
            self._get_net_uuid()
            # TODO 可以添加其他机器标识
        ])
        return mc


    def _get_disk_uuid(self):
        uuid = ''
        if self._is_windows() :
            uuid = self._get_win_disk_uuid()

        else :
            uuid = self._get_linux_disk_uuid()
        return uuid


    def _get_win_disk_uuid(self):
        '''
        获取硬盘唯一标识。
        tuple = win32api.GetVolumeInformation(path)

            Parameters
                path : string
                        The root path of the volume on which information is being requested.
            Return Value
                The return is a tuple of:
                    string - Volume Name
                    long - Volume serial number.
                    long - Maximum Component Length of a file name.
                    long - Sys Flags - other flags specific to the file system. See the api for details.
                    string - File System Name
        '''
        import win32api
        disk_infos = win32api.GetVolumeInformation("C:\\")
        s_disk_infos = list(map(lambda e: str(e), disk_infos))
        return ';'.join(s_disk_infos[1:])   # 排除最容易修改的分卷名，其他参数除非格盘或重装系统，否则难以改变


    def _get_linux_disk_uuid(self):
        # TODO 因权限问题、或受限于系统组件，尚未有较好方法获取硬盘唯一标识
        return ''


    def _get_net_uuid(self):
        uuid = ''
        if self._is_windows() :
            uuid = self._get_win_net_uuid()

        else :
            uuid = self._get_linux_net_uuid()
        return uuid

    
    def _get_win_net_uuid(self):
        # FIXME win 网卡比较容易改变（例如安装虚拟机等都会新增网卡），暂不纳入标识
        return ''


    def _get_linux_net_uuid(self):
        '''
        获取网卡唯一标识（MAC 地址）
        '''
        macs = []
        for k, v in net_if_addrs().items():
            for item in v:
                address = item[1]
                if len(address) != 17 :
                    continue
                if ('-' in address) or (':' in address) :
                    macs.append(address)
        return ';'.join(macs)


    def _os(self) :
        return platform.system()


    def _is_windows(self) :
        return self._os() == 'Windows'


    def _is_mac(self) :
        return self._os() == 'Darwin'


    def _is_linux(self) :
        return self._os() == 'Linux'

